/// <reference name "MicrosoftAjax.js" assembly="System.Web.Extensions" />

(function ($) 
{
	/*This class is capable of creation of a special image popup in the popupArea, according to settings information. 
	The popup contains an image.
	Container area should contain a trigger element for the popup and a link, which will be the source for the image.
	This class requires jQuery.domec plugin. Use it like $('#container).imagePopup().*/
	ImagePopup = function(containerArea, popupArea, settings)
	{
		this.hideDelayTimer = null;
		this.trigger = $(settings.triggerSelector, containerArea);
		this.link = $(settings.linkSelector, containerArea);
		this.settings = settings;
		this.popupArea = popupArea;
		
		if(!this.trigger || !this.link || this.link.length == 0 || !this.link[0].href || !this.link[0].innerHTML)
		{
			return;
		}
		
		this.trigger.mouseover(Function.createDelegate(this, this.handleMouseOver));
		this.trigger.mouseout(Function.createDelegate(this, this.handleMouseOut));
		
		this.popup = null;
		this.hideDelayTimer = null;
		this.showDelayTimer = null;
		
		this.mouseEventArgs = null;
	};
		
	ImagePopup.prototype = 
	{
		handleMouseOver : function(e)
		{
			if(!this.popup)
			{
				this.createPopup();
			}

			if (this.hideDelayTimer) 
			{
				clearTimeout(this.hideDelayTimer);
			}

			if (this.popup.is(':animated, :visible'))
			{
				return;
			}
			
			this.mouseEventArgs = e;
			
			//this.showDelayTimer is set to null in handleMouseOut
			if(!this.showDelayTimer)
			{
				this.showDelayTimer = setTimeout(Function.createDelegate(this, this.showPopup), this.settings.showDelay);
			}
		},
		
		showPopup : function()
		{
			var startTopPosition;
			var startLeftPosition;
			
			if(this.settings.useMousePosition && this.mouseEventArgs)
			{
			startTopPosition = this.mouseEventArgs.clientY;
			startLeftPosition = this.mouseEventArgs.clientX;
			}
			else
			{
			startTopPosition = this.trigger.offset().top;
			startLeftPosition = this.trigger.offset().left;
			}
			startTopPosition += this.settings.top;
			startLeftPosition += this.settings.left;

			var endTopPosition = startTopPosition - this.settings.showDistance;
			
			this.popup.css(
			{
			  //Position should be set to absolute to make the simultaneous rendering of different popups independent.
				position: 'absolute',
				display: 'block',
				//opacity: 0, //opacity doesn't work in IE
				top: startTopPosition,
				left: startLeftPosition
			}).animate(
			{
				//opacity: 1,
				top: endTopPosition
			}, this.settings.showTime);
		},
		
		handleMouseOut : function()
		{
			if(!this.popup)
			{
				return;
			}
			if (this.hideDelayTimer)
			{
				clearTimeout(this.hideDelayTimer);
			}
			if(this.showDelayTimer)
			{
			  clearTimeout(this.showDelayTimer);
			  this.showDelayTimer = null; //as we'll check in handleMouseOver
			}

			this.hideDelayTimer = setTimeout(Function.createDelegate(this, this.hidePopup), this.settings.hideDelay);
		},
		
		hidePopup : function()
		{
			this.hideDelayTimer = null;
			this.popup.animate(
			{
				//opacity: 0,
				top: '-=' + this.settings.hideDistance + 'px'
			}, this.settings.hideTime, 
			  Function.createDelegate(this, function()
			    {
			      this.popup.css('display', 'none');
			    }
			  )
			);
			
		},
		
		createPopup : function()
		{
			this.popup = $.create('div', {'class': this.settings.imageAreaClass}, 
				[$.create('img', 
					{
						'class' : this.settings.imageClass, 
						'src' : this.link[0].href, 
						'alt' : this.link[0].innerHTML
					})
				]);
			
			this.popup.mouseover(Function.createDelegate(this, this.handleMouseOver));
			this.popup.mouseout(Function.createDelegate(this, this.handleMouseOut));
			this.popupArea.append(this.popup);
			
			//Do not hide the popup before the png fix call and do not call png fix before appending the popup to the dom tree,
			//as in these cases $(image).width() returns 0. Png fix uses this value and renders png images improperly.
			this.popup.pngFixInner(); //fix inner images
			this.popup.hide();
		},
		
		defaults : 
		{
			triggerSelector : '.trigger',
			linkSelector : '.link',
			showDistance : 10,
			hideDistance : 20,
			showTime : 250,
			hideTime : 250,
			showDelay : 500,
			hideDelay : 400,
			left : 20,
			top : -140,
			imageClass : 'thumbnail',
			imageAreaClass: 'thumbnail-area',
			useMousePosition : true
		}
	};

	$.fn.imageBubble = function (options) 
	{
		var settings = $.extend({}, ImagePopup.prototype.defaults, options);
		//Overflow should be set to visible to allow displaying image popups
		//Position should be set to absolute to exclude the div from the normal flow of the browser formatting 
		//(IE works slowly otherwise and displays incorrectly)
		var popupArea = $.create('div', {'style' : 'overflow: visible; position:absolute; top:0px; left:0px;'});
		$('body').append(popupArea);
		//Here "this" is the jQuery selection set, over which the imageBubble function was called.
		return this.each(function () 
		{
			//Here "this" is the DOM child element of the selection set, over which the imageBubble function was called.
			var imagePopup = new ImagePopup(this, popupArea, settings);
		});
	}
})(jQuery);